@@ -2,6 +2,7 @@ from optuna.samplers import nsgaii
 from optuna.samplers._base import BaseSampler
 from optuna.samplers._brute_force import BruteForceSampler
 from optuna.samplers._cmaes import CmaEsSampler
+from optuna.samplers._gp.sampler import GPSampler
 from optuna.samplers._grid import GridSampler
 from optuna.samplers._nsgaiii._sampler import NSGAIIISampler
 from optuna.samplers._partial_fixed import PartialFixedSampler
@@ -27,6 +28,7 @@ __all__ = [
     "QMCSampler",
     "RandomSampler",
     "TPESampler",
+    "GPSampler",
     "intersection_search_space",
     "nsgaii",
 ]
