@@ -7,7 +7,6 @@ from typing import cast
 from typing import Dict
 from typing import Optional
 from typing import Sequence
-from typing import Union
 import warnings
 
 import numpy as np
@@ -29,6 +28,7 @@ from optuna.search_space import IntersectionSearchSpace
 from optuna.search_space.group_decomposed import _GroupDecomposedSearchSpace
 from optuna.search_space.group_decomposed import _SearchSpaceGroup
 from optuna.study import Study
+from optuna.study._multi_objective import _fast_non_dominated_sort
 from optuna.study._study_direction import StudyDirection
 from optuna.trial import FrozenTrial
 from optuna.trial import TrialState
@@ -80,7 +80,7 @@ class TPESampler(BaseSampler):
     For multi-objective TPE (MOTPE), please refer to the following papers:
 
     - `Multiobjective Tree-Structured Parzen Estimator for Computationally Expensive Optimization
-      Problems <https://dl.acm.org/doi/10.1145/3377930.3389817>`_
+      Problems <https://doi.org/10.1145/3377930.3389817>`_
     - `Multiobjective Tree-Structured Parzen Estimator <https://doi.org/10.1613/jair.1.13188>`_
 
     Example:
@@ -161,7 +161,7 @@ class TPESampler(BaseSampler):
                 <https://papers.nips.cc/paper/4443-algorithms-for-hyper-parameter-optimization.pdf>`_
                 ). The weights of good trials, i.e., trials to construct `l(x)`, are computed by a
                 rule based on the hypervolume contribution proposed in the `paper of MOTPE
-                <https://dl.acm.org/doi/10.1145/3377930.3389817>`_.
+                <https://doi.org/10.1613/jair.1.13188>`_.
         seed:
             Seed for random number generator.
         multivariate:
@@ -477,59 +477,78 @@ class TPESampler(BaseSampler):
             self._constraints_func is not None,
         )
 
-        below = self._get_internal_repr(below_trials, search_space)
-        above = self._get_internal_repr(above_trials, search_space)
+        mpe_below = self._build_parzen_estimator(
+            study, search_space, below_trials, handle_below=True
+        )
+        mpe_above = self._build_parzen_estimator(
+            study, search_space, above_trials, handle_below=False
+        )
 
-        # We then sample by maximizing log likelihood ratio.
-        if study._is_multi_objective():
-            param_mask_below = []
-            for trial in below_trials:
-                param_mask_below.append(
-                    all((param_name in trial.params) for param_name in search_space)
-                )
-            weights_below = _calculate_weights_below_for_multi_objective(
-                study, below_trials, self._constraints_func
-            )[param_mask_below]
-            mpe_below = _ParzenEstimator(
-                below, search_space, self._parzen_estimator_parameters, weights_below
-            )
-        else:
-            mpe_below = _ParzenEstimator(below, search_space, self._parzen_estimator_parameters)
-        mpe_above = _ParzenEstimator(above, search_space, self._parzen_estimator_parameters)
         samples_below = mpe_below.sample(self._rng.rng, self._n_ei_candidates)
-        log_likelihoods_below = mpe_below.log_pdf(samples_below)
-        log_likelihoods_above = mpe_above.log_pdf(samples_below)
-        ret = TPESampler._compare(samples_below, log_likelihoods_below, log_likelihoods_above)
+        acq_func_vals = self._compute_acquisition_func(samples_below, mpe_below, mpe_above)
+        ret = TPESampler._compare(samples_below, acq_func_vals)
 
         for param_name, dist in search_space.items():
             ret[param_name] = dist.to_external_repr(ret[param_name])
 
         return ret
 
+    def _build_parzen_estimator(
+        self,
+        study: Study,
+        search_space: dict[str, BaseDistribution],
+        trials: list[FrozenTrial],
+        handle_below: bool,
+    ) -> _ParzenEstimator:
+        observations = self._get_internal_repr(trials, search_space)
+        if handle_below and study._is_multi_objective():
+            param_mask_below = []
+            for trial in trials:
+                param_mask_below.append(
+                    all((param_name in trial.params) for param_name in search_space)
+                )
+            weights_below = _calculate_weights_below_for_multi_objective(
+                study, trials, self._constraints_func
+            )[param_mask_below]
+            mpe = _ParzenEstimator(
+                observations, search_space, self._parzen_estimator_parameters, weights_below
+            )
+        else:
+            mpe = _ParzenEstimator(observations, search_space, self._parzen_estimator_parameters)
+
+        return mpe
+
+    def _compute_acquisition_func(
+        self,
+        samples: dict[str, np.ndarray],
+        mpe_below: _ParzenEstimator,
+        mpe_above: _ParzenEstimator,
+    ) -> np.ndarray:
+        log_likelihoods_below = mpe_below.log_pdf(samples)
+        log_likelihoods_above = mpe_above.log_pdf(samples)
+        acq_func_vals = log_likelihoods_below - log_likelihoods_above
+        return acq_func_vals
+
     @classmethod
     def _compare(
         cls,
         samples: Dict[str, np.ndarray],
-        log_l: np.ndarray,
-        log_g: np.ndarray,
-    ) -> Dict[str, Union[float, int]]:
+        acquisition_func_vals: np.ndarray,
+    ) -> dict[str, int | float]:
         sample_size = next(iter(samples.values())).size
-        if sample_size:
-            score = log_l - log_g
-            if sample_size != score.size:
-                raise ValueError(
-                    "The size of the 'samples' and that of the 'score' "
-                    "should be same. "
-                    "But (samples.size, score.size) = ({}, {})".format(sample_size, score.size)
-                )
-            best = np.argmax(score)
-            return {k: v[best].item() for k, v in samples.items()}
-        else:
+        if sample_size == 0:
+            raise ValueError(f"The size of `samples` must be positive, but got {sample_size}.")
+
+        if sample_size != acquisition_func_vals.size:
             raise ValueError(
-                "The size of 'samples' should be more than 0."
-                "But samples.size = {}".format(sample_size)
+                "The sizes of `samples` and `acquisition_func_vals` must be same, but got "
+                "(samples.size, acquisition_func_vals.size) = "
+                f"({sample_size}, {acquisition_func_vals.size})."
             )
 
+        best_idx = np.argmax(acquisition_func_vals)
+        return {k: v[best_idx].item() for k, v in samples.items()}
+
     @staticmethod
     def hyperopt_parameters() -> Dict[str, Any]:
         """Return the the default parameters of hyperopt (v0.1.2).
@@ -589,21 +608,6 @@ class TPESampler(BaseSampler):
         self._random_sampler.after_trial(study, trial, state, values)
 
 
-def _calculate_nondomination_rank(loss_vals: np.ndarray, n_below: int) -> np.ndarray:
-    ranks = np.full(len(loss_vals), -1)
-    num_ranked = 0
-    rank = 0
-    domination_mat = np.all(loss_vals[:, None, :] >= loss_vals[None, :, :], axis=2) & np.any(
-        loss_vals[:, None, :] > loss_vals[None, :, :], axis=2
-    )
-    while num_ranked < n_below:
-        counts = np.sum((ranks == -1)[None, :] & domination_mat, axis=1)
-        num_ranked += np.sum((counts == 0) & (ranks == -1))
-        ranks[(counts == 0) & (ranks == -1)] = rank
-        rank += 1
-    return ranks
-
-
 def _split_trials(
     study: Study,
     trials: list[FrozenTrial],
@@ -616,14 +620,16 @@ def _split_trials(
     infeasible_trials = []
 
     for trial in trials:
-        if constraints_enabled and _get_infeasible_trial_score(trial) > 0:
+        if trial.state == TrialState.RUNNING:
+            # We should check if the trial is RUNNING before the feasibility check
+            # because its constraint values have not yet been set.
+            running_trials.append(trial)
+        elif constraints_enabled and _get_infeasible_trial_score(trial) > 0:
             infeasible_trials.append(trial)
         elif trial.state == TrialState.COMPLETE:
             complete_trials.append(trial)
         elif trial.state == TrialState.PRUNED:
             pruned_trials.append(trial)
-        elif trial.state == TrialState.RUNNING:
-            running_trials.append(trial)
         else:
             assert False
 
@@ -675,13 +681,11 @@ def _split_complete_trials_multi_objective(
         # The type of trials must be `list`, but not `Sequence`.
         return [], list(trials)
 
-    lvals = np.asarray([trial.values for trial in trials])
-    for i, direction in enumerate(study.directions):
-        if direction == StudyDirection.MAXIMIZE:
-            lvals[:, i] *= -1
+    lvals = np.array([trial.values for trial in trials])
+    lvals *= np.array([-1.0 if d == StudyDirection.MAXIMIZE else 1.0 for d in study.directions])
 
     # Solving HSSP for variables number of times is a waste of time.
-    nondomination_ranks = _calculate_nondomination_rank(lvals, n_below)
+    nondomination_ranks = _fast_non_dominated_sort(lvals, n_below=n_below)
     assert 0 <= n_below <= len(lvals)
 
     indices = np.array(range(len(lvals)))
