@@ -72,37 +72,78 @@ def plot_timeline(study: Study) -> "go.Figure":
             their lifetime.
 
     Returns:
-        A :class:`plotly.graph_objs.Figure` object.
+        A :class:`plotly.graph_objects.Figure` object.
     """
     _imports.check()
     info = _get_timeline_info(study)
     return _get_timeline_plot(info)
 
 
+def _get_max_datetime_complete(study: Study) -> datetime.datetime:
+    max_run_duration = max(
+        [
+            t.datetime_complete - t.datetime_start
+            for t in study.trials
+            if t.datetime_complete is not None and t.datetime_start is not None
+        ],
+        default=None,
+    )
+    if _is_running_trials_in_study(study, max_run_duration):
+        return datetime.datetime.now()
+
+    return max(
+        [t.datetime_complete for t in study.trials if t.datetime_complete is not None],
+        default=datetime.datetime.now(),
+    )
+
+
+def _is_running_trials_in_study(study: Study, max_run_duration: datetime.timedelta | None) -> bool:
+    running_trials = study.get_trials(states=(TrialState.RUNNING,), deepcopy=False)
+    if max_run_duration is None:
+        return len(running_trials) > 0
+
+    now = datetime.datetime.now()
+    # This heuristic is to check whether we have trials that were somehow killed,
+    # still remain as `RUNNING` in `study`.
+    return any(
+        now - t.datetime_start < 5 * max_run_duration
+        for t in running_trials
+        # MyPy redefinition: Running trial should have datetime_start.
+        if t.datetime_start is not None
+    )
+
+
 def _get_timeline_info(study: Study) -> _TimelineInfo:
     bars = []
-    for t in study.get_trials(deepcopy=False):
-        date_complete = t.datetime_complete or datetime.datetime.now()
-        date_start = t.datetime_start or date_complete
+
+    max_datetime = _get_max_datetime_complete(study)
+    timedelta_for_small_bar = datetime.timedelta(seconds=1)
+    for trial in study.get_trials(deepcopy=False):
+        datetime_start = trial.datetime_start or max_datetime
+        datetime_complete = (
+            max_datetime + timedelta_for_small_bar
+            if trial.state == TrialState.RUNNING
+            else trial.datetime_complete or datetime_start + timedelta_for_small_bar
+        )
         infeasible = (
             False
-            if _CONSTRAINTS_KEY not in t.system_attrs
-            else any([x > 0 for x in t.system_attrs[_CONSTRAINTS_KEY]])
+            if _CONSTRAINTS_KEY not in trial.system_attrs
+            else any([x > 0 for x in trial.system_attrs[_CONSTRAINTS_KEY]])
         )
-        if date_complete < date_start:
+        if datetime_complete < datetime_start:
             _logger.warning(
                 (
-                    f"The start and end times for Trial {t.number} seem to be reversed. "
-                    f"The start time is {date_start} and the end time is {date_complete}."
+                    f"The start and end times for Trial {trial.number} seem to be reversed. "
+                    f"The start time is {datetime_start} and the end time is {datetime_complete}."
                 )
             )
         bars.append(
             _TimelineBarInfo(
-                number=t.number,
-                start=date_start,
-                complete=date_complete,
-                state=t.state,
-                hovertext=_make_hovertext(t),
+                number=trial.number,
+                start=datetime_start,
+                complete=datetime_complete,
+                state=trial.state,
+                hovertext=_make_hovertext(trial),
                 infeasible=infeasible,
             )
         )
@@ -123,15 +164,15 @@ def _get_timeline_plot(info: _TimelineInfo) -> "go.Figure":
     }
 
     fig = go.Figure()
-    for s in sorted(TrialState, key=lambda x: x.name):
-        if s.name == "COMPLETE":
-            infeasible_bars = [b for b in info.bars if b.state == s and b.infeasible]
-            feasible_bars = [b for b in info.bars if b.state == s and not b.infeasible]
+    for state in sorted(TrialState, key=lambda x: x.name):
+        if state.name == "COMPLETE":
+            infeasible_bars = [b for b in info.bars if b.state == state and b.infeasible]
+            feasible_bars = [b for b in info.bars if b.state == state and not b.infeasible]
             _plot_bars(infeasible_bars, "#cccccc", "INFEASIBLE", fig)
-            _plot_bars(feasible_bars, _cm[s.name], s.name, fig)
+            _plot_bars(feasible_bars, _cm[state.name], state.name, fig)
         else:
-            bars = [b for b in info.bars if b.state == s]
-            _plot_bars(bars, _cm[s.name], s.name, fig)
+            bars = [b for b in info.bars if b.state == state]
+            _plot_bars(bars, _cm[state.name], state.name, fig)
     fig.update_xaxes(type="date")
     fig.update_layout(
         go.Layout(
